#!/bin/bash
export LANG=en_US.UTF-8
_stage=$1
_stage_lock="/root/initial/tasks/${_stage}.done"
_prov_type=$(dmidecode -s system-product-name)
_centos_release=$(/bin/rpm --eval='%centos_ver')
if [ "${_prov_type}" == 'OpenStack Nova' ]; then
  _prov_type=openstack
else
  _prov_type=kickstart
fi

stage1() {
    check_stage
    if [ "${_prov_type}" == 'openstack' ]; then
      sed -i '1 i\MAILTO=""' /var/spool/cron/root
      touch /var/cpanel/activate/features/disable_feature_showcase
      touch /etc/.whostmgrft
      sed -i '\#/dev/shm#d' /etc/fstab
      cat >> /etc/fstab <<-FSTAB_EOF
	tmpfs   /dev/shm        tmpfs   rw,noexec,nosuid,noatime,nodev,mode=711,size=500M       0       0
	tmpfs   /ramdisk        tmpfs   defaults,noatime,nosuid,nodev,size=4096m                0       0
	none    /var/spool/exim tmpfs   rw,noexec,nosuid,nodev,uid=47,gid=12,mode=750           0       0
	FSTAB_EOF
  
      ## Creates directories and starts needed services for multihome
      mkdir -vp /{admin,ramdisk}
      curl -s http://kickstart.hostgator.com/hgstartprovo > /etc/init.d/hgstartprovo
      mount -a
      chmod +x /etc/init.d/hgstartprovo
      chkconfig hgstartprovo on
      service hgstartprovo start
      yum install epel-release -y
      yum install https://yum.puppetlabs.com/puppetlabs-release-el-6.noarch.rpm -y
      yum install puppet -y
    fi

    sed -i "s#ETHDEV.*#ETHDEV $(ip route show default 0.0.0.0/0 | grep -Po '(?<=dev )\S+')#" /etc/wwwacct.conf
    /usr/bin/curl -s "http://nas.hostgator.com/assign.php?server=$(uname -n)"
    if [ "${_centos_release}" -eq 7 ]; then
      timedatectl set-timezone "${TIME_ZONE}"
      timedatectl set-local-rtc 0
      timedatectl set-ntp true
      _mysql_service=mysqld
    else
      rm -vf /etc/localtime
      ln -vs "/usr/share/zoneinfo/${TIME_ZONE}" /etc/localtime
      _mysql_service=mysql
    fi
    sed -i '/mysql/d' /etc/chkserv.d/chkservd.conf
    /scripts/setupftpserver pure-ftpd
    service cpanel stop
    service ${_mysql_service} stop
    mkdir -vp /opt/eig_linux/etc/puppetfeatures
    touch /opt/eig_linux/etc/puppetfeatures/{no_custom_jaillshell,skip_sm}
    rm -vf {/var/cpanel/globalcache/cpanel.cache,/root/.cpanel/datastore/MOUNTS_CACHE__proc_mounts}
     FACTER_foreman_organization=${BRAND} FACTER_foreman_location=${LOCATION} puppet agent -t --server hensonm.hostgator.com --ca_server hensonca.hostgator.com --tags foreman_registration
    while [ $? -eq 1 ]; do
        sleep 5
         FACTER_foreman_organization=${BRAND} FACTER_foreman_location=${LOCATION} puppet agent -t --server hensonm.hostgator.com --ca_server hensonca.hostgator.com --tags foreman_registration
    done
    touch "${_stage_lock}"
}

stage2() {
    check_stage
    puppet agent --enable
     puppet agent -t --server hensonm.hostgator.com --ca_server hensonca.hostgator.com --tags=yum_repos,puppet_agent,resolvers
    while [ $? -eq 1 ]; do
        sleep 5
         puppet agent -t --server hensonm.hostgator.com --ca_server hensonca.hostgator.com --tags=yum_repos,puppet_agent,resolvers
    done
    puppet agent --disable 'provisioning in progress'
    yum clean all -y --enablerepo=*
    yum makecache
    if [ -f /etc/cpanel/ea4/is_ea4 ]; then
        if [ "${_centos_release}" -eq 7 ]; then
          sed -i '/JoinControllers/c\JoinControllers=blkio,cpu,cpuacct,cpuset,memory' /etc/systemd/system.conf
          source /root/initial/ea4_c7_rpms
	  _perc_repo=eig-percona57
	  rm -vfr /var/lib/mysql
          mkdir -vp /etc/systemd/system/{rescue.service.d,httpd.service.d,cpanel.service.d,dovecot.service.d,mysqld.service.d,pure-authd.service.d}
	  touch /opt/eig_linux/etc/puppetfeatures/esop17141
        else
          source /root/initial/ea4_c6_rpms
          _perc_repo=hgpercona56
        fi
        mkdir -vp /etc/cpanel/ea4/profiles/custom
    else
        source /root/initial/rpms
        _perc_repo=hgpercona56
    fi
    brand_rpms=${BRAND}_rpms
    prov_rpms=${_prov_type}_rpms
    prod_rpms=${PRODUCT}_rpms
    yum -y remove ${rm_rpms}
    yum -y --enablerepo=${_perc_repo} --enablerepo=hgrepo-testing --enablerepo=epel --disableexcludes=main install ${common_rpms} ${!prov_rpms} ${!prod_rpms} ${!brand_rpms} || exit 1
    touch "${_stage_lock}"
}

stage3() {
    check_stage
    chkconfig datacycle on
    chkconfig eig-firewall on
    service ipaliases start
    touch /etc/mysqlupdisable
    touch /etc/ftpupdisable
    if [ "${_centos_release}" -eq 7 ]; then
      systemctl daemon-reload
      rm -vf /root/.my.cnf
      mysqld --initialize-insecure --user=mysql
    fi
    service mysql status &> /dev/null || service mysql start
    /scripts/upcp --sync
    service mysql status &> /dev/null || service mysql start
    /scripts/securemysql -F
    /scripts/mysqlconnectioncheck
    while ! mysqladmin status; do
      sleep 5
    done
    mysql -BNe "uninstall plugin validate_password"
    sed -i '/\(upcp --cron\|optimizefs\|update_db_cache\)/d' /var/spool/cron/root
    if [ "${PRODUCT}" != 'owp' ]; then
      ### Add postgres for BH ESOP-2655
      if [ "${BRAND}" == "BLUEHOST" ]; then
          echo 'y' | /scripts/installpostgres
          /opt/eig_linux/bin-legacy/fixpostgres
      fi
      curl -vks http://files.softaculous.com/install.sh > /root/install_softac.sh
      chmod 755 /root/install_softac.sh
      /root/install_softac.sh &>> /root/initial/softaculous_install.log
      rm -f /root/install_softac.sh
      touch /opt/eig_linux/etc/puppetfeatures/softaculous
      chmod 4555  /usr/local/cpanel/whostmgr/docroot/cgi/softaculous/bin/soft
    fi
    whmapi1 disable_cpgreylist
    puppet agent --enable
    puppet agent -t
    while [ $? -eq 1 ]; do
        sleep 5
         puppet agent -t 
    done
    service mysql status &> /dev/null || service mysql start
    puppet agent -t
    service mysql status &> /dev/null || service mysql start
    puppet agent -t
    rm -vf /var/cpanel/ssl/*-{CRTINFO,NOT_AFTER,CN,SIGNATURE_CHAIN_VERIFIED,DOMAINS}
    /usr/local/cpanel/bin/checkallsslcerts
    /scripts/install_lets_encrypt_autossl_provider
    whmapi1 set_autossl_provider provider=LetsEncrypt x_terms_of_service_accepted=https://letsencrypt.org/documents/LE-SA-v1.2-November-15-2017.pdf
    whmapi1 set_autossl_metadata metadata_json='{"clobber_externally_signed":1, "notify_autossl_expiry":0, "notify_autossl_expiry_coverage":0, "notify_autossl_renewal":0, "notify_autossl_renewal_coverage":0, "notify_autossl_renewal_coverage_reduced":0, "notify_autossl_renewal_uncovered_domains":0, "notify_autossl_expiry_coverage_user":0, "notify_autossl_expiry_user":0, "notify_autossl_renewal_coverage_reduced_user":0, "notify_autossl_renewal_coverage_user":0, "notify_autossl_renewal_uncovered_domains_user":0 }'
    /opt/hgmods/multi_shared.pl
    touch /var/cpanel/diya_allow
    rm -vf /admin/MIRRORING.FROM
    yum -y --enablerepo=endurance-419-kernel-testing update
    yum -y install --enablerepo=endurance-419-kernel-testing kernel-eiguserproc
    chkconfig fastmail off
    /scripts/fixquotas
    mysql_upgrade

    ## Add Backups
    _min=$(shuf -i 0-59 -n 1)
    _hour=$(shuf -i 0-23 -n 1)
    _day=$( head -1 <(grep -oa '[0-1]\|[5-6]' /dev/urandom) )

    echo "${_min} ${_hour} * * ${_day} cd /admin/hgbackupdir/; ./backup.pl --full" >> /var/spool/cron/root
    echo "${_min} ${_hour} * * 3 cd /admin/hgbackupdir/; ./backup.pl --innodb" >> /var/spool/cron/root

    ### custapi randomly drops requests causing it to fail. Adding this as a work around
    /opt/hgmods/hal_accesshash.pl
    while [ $? -eq 1 ]; do
        sleep 5
        /opt/hgmods/hal_accesshash.pl
    done
    sed -i -r '/\/(home[1-9]?|mysql)/s/[0-9]$/0/' /etc/fstab
    sed -i "\#/var/mysqltmp#a /tmp    /var/tmp        none    bind,defaults,noexec,noatime,nosuid,nodev 0 0" /etc/fstab
    touch "${_stage_lock}"
}

stage4() {
    check_stage
    #cp_update
    if [ -f /etc/cpanel/ea4/is_ea4 ]; then
        _phpver=ea-php74
        if [ "${PRODUCT}" == 'owp' ]; then
          _php_fpm=1
        fi
        /usr/local/bin/ea_install_profile --install /etc/cpanel/ea4/profiles/custom/ea4_eig_profile.json || exit 1
        _php_vers=( $(/bin/awk -F: '/suphp/ {print $1}' <(/usr/local/cpanel/bin/rebuild_phpconf --available)) )
        /usr/local/cpanel/bin/rebuild_phpconf --default=${_phpver}$(printf -- ' --%s=suphp' "${_php_vers[@]}") || exit 1
        if [ "${_php_fpm}" == 1 ]; then
          whmapi1 php_set_default_accounts_to_fpm default_accounts_to_fpm=1
          /scripts/php_fpm_config --convert_all --noprompt
        else
          whmapi1 php_set_default_accounts_to_fpm default_accounts_to_fpm=0
	fi
    fi
    puppet agent -t
    puppet agent -t
    rm -vf /var/cpanel/conf/apache/local
    /scripts/rebuildhttpdconf
    if [ "${_centos_release}" -eq 7 ]; then
      groupadd -r nohidepid
      usermod -a -G nohidepid polkitd
      usermod -a -G nohidepid zabbix
    fi

    if [ -e "/opt/eig_linux/etc/cluster_defaults.conf" ]
    then
        echo "setup cmh obj redis permissions"
        source /opt/eig_linux/etc/cluster_defaults.conf
        ROOTPASS=$(/usr/bin/redis-cli --user "${obj_default_username}" --pass "${obj_default_password}" -h ${obj_vip} ACL GENPASS)
        MON_PASS=$(/usr/bin/redis-cli --user "${obj_default_username}" --pass "${obj_default_password}" -h ${obj_vip} ACL GENPASS)
        if [[ "z$ROOTPASS" == "z" ]]
        then
            echo "received empty string when attempting to create root user"
        else
            # create config file
            cat > /root/redis-root.conf <<ROOTEOF
REDIS_USER="root"
REDIS_PASS="${ROOTPASS}"
REDIS_HOST="${obj_vip}"
REDIS_SERVERS="${obj_servers}"
ROOTEOF
            cat > /nfssys/etc/redis_monitoring.conf <<MONEOF
REDIS_USER="__sys__monitoring"
REDIS_PASS="${MON_PASS}"
REDIS_HOST="${obj_vip}"
REDIS_SERVERS="${obj_servers}"
MONEOF
            /usr/bin/chmod 0400 /nfssys/etc/redis_monitoring.conf
            /usr/bin/chown zabbix.zabbix /nfssys/etc/redis_monitoring.conf
            # create ACL files
            ACL_ADD_FILE=$(mktemp)
            cat >${ACL_ADD_FILE} <<ADDEOF
ACL SETUSER root on ~* +@all >${ROOTPASS}
ACL SETUSER __sys__monitoring on ~* +info >${MON_PASS}
ACL SAVE
ADDEOF
            ACL_DEL_FILE=$(mktemp)
            cat >${ACL_DEL_FILE} <<DELEOF
ACL DELUSER ${obj_default_username}
ACL SAVE
DELEOF
            # setup ACLs
            for objserver in ${obj_servers}
            do
                cat ${ACL_ADD_FILE} |/usr/bin/redis-cli --user "${obj_default_username}" --pass "${obj_default_password}" -h ${objserver}
                cat ${ACL_DEL_FILE} |/usr/bin/redis-cli --user root --pass "${ROOTPASS}" -h ${objserver}
            done
            /bin/rm -vf ${ACL_ADD_FILE} ${ACL_DEL_FILE}
        fi
	yum remove eig-site247-wp -y
	/root/bin/rmysql_profile_gen.pl
	/usr/local/sbin/gt cluster_db_setup
	yum --enablerepo=hgrepo-testing install eig-site247-wp -y
    fi

    yum -y remove initial-scripts-${BRAND}
    echo "Rebooting in 2 Minutes"
    echo "shutdown -rf now" | at now + 2 min
    touch "${_stage_lock}"
}

cp_update() {
    if ! grep -q '11.58' /usr/local/cpanel/version; then
        touch /opt/eig_linux/etc/puppetfeatures/cpupdate1158
         puppet agent -t --tags=cpanel
         puppet agent -t --tags=cpanel
        /scripts/upcp
    fi
}

check_stage() {
    if [ -f "${_stage_lock}" ]; then
        exit 0
    fi
    echo "Running ${_stage}"
}

${_stage}
